(*^

::[	Information =

	"This is a Mathematica Notebook file.  It contains ASCII text, and can be
	transferred by email, ftp, or other text-file transfer utility.  It should
	be read or edited using a copy of Mathematica or MathReader.  If you 
	received this as email, use your mail application or copy/paste to save 
	everything from the line containing (*^ down to the line containing ^*)
	into a plain text file.  On some systems you may have to give the file a 
	name ending with ".ma" to allow Mathematica to recognize it as a Notebook.
	The line below identifies what version of Mathematica created this file,
	but it can be opened using any other version as well.";

	FrontEndVersion = "X Window System Mathematica Notebook Front End Version 2.2";

	X11StandardFontEncoding; 
	
	fontset = title, inactive, noPageBreakBelow, noPageBreakInGroup, nohscroll, preserveAspect, groupLikeTitle, center, M7, bold, e8,  24, fontName, "times";
	fontset = subtitle, inactive, noPageBreakBelow, noPageBreakInGroup, nohscroll, preserveAspect, groupLikeTitle, center, M7, bold, e6,  18, fontName, "times";
	fontset = subsubtitle, inactive, noPageBreakBelow, noPageBreakInGroup, nohscroll, preserveAspect, groupLikeTitle, center, M7, italic, e6,  14, fontName, "times";
	fontset = section, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, grayBox, M22, bold, a20,  18, fontName, "times";
	fontset = subsection, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, blackBox, M19, bold, a15,  14, fontName, "times";
	fontset = subsubsection, inactive, noPageBreakBelow, nohscroll, preserveAspect, groupLikeSection, whiteBox, M18, bold, a12,  12, fontName, "times";
	fontset = text, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";
	fontset = smalltext, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  10, fontName, "times";
	fontset = input, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeInput, M42, N23, bold,  12, fontName, "courier";
	fontset = output, output, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23, L-5,  12, fontName, "courier";
	fontset = message, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23,  12, fontName, "courier";
	fontset = print, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23,  12, fontName, "courier";
	fontset = info, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeOutput, M42, N23,  12, fontName, "courier";
	fontset = postscript, PostScript, formatAsPostScript, output, inactive, noPageBreakInGroup, nowordwrap, preserveAspect, groupLikeGraphics, M7, l34, w282, h287,  12, fontName, "courier";
	fontset = name, inactive, noPageBreakInGroup, nohscroll, preserveAspect, M7, italic, B65535,  10, fontName, "times";
	fontset = header, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7, italic,  12, fontName, "times";
	fontset = leftheader,  12, fontName, "times";
	fontset = footer, inactive, nohscroll, noKeepOnOnePage, preserveAspect, center, M7, italic,  12, fontName, "times";
	fontset = leftfooter,  12, fontName, "times";
	fontset = help, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";
	fontset = clipboard, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";
	fontset = completions, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "courier";
	fontset = special1, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";
	fontset = special2, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";
	fontset = special3, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";
	fontset = special4, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";
	fontset = special5, inactive, nohscroll, noKeepOnOnePage, preserveAspect, M7,  12, fontName, "times";paletteColors = 128; automaticGrouping; currentKernel; 
]
:[font = title; inactive; locked; preserveAspect; startGroup]
Jackson Networks
:[font = input; locked; initialization; closed; preserveAspect]
*)
<<digraph.m
(*
:[font = section; inactive; locked; preserveAspect; startGroup]
Input Network Information
:[font = text; inactive; locked; preserveAspect]
The following two examples illustrate how to implement various types of Jackson networks.  Input for the notebook consists of four parts.  The first one is the network connectivity matrix.  Element on the i-th row and j-th column in the matrix represents the probability of the output from node i to node j. This matrix informs mathematica as to how the network nodes are connected.  The second one is a vector which represents the external rates into the network.  The third one itemizes the server departure rate for each node in the network.  Finally, the fourth one lists how many servers are at each node in the network. 
 
Note if |matrix| = n x n, then |vector| = n.
:[font = subsection; inactive; locked; preserveAspect; startGroup]
Example 1 -
:[font = text; inactive; locked; preserveAspect]
This is an example of a Jackson network with numerical parameters.  When the notebook is first opened this cell is initialized.
:[font = input; locked; initialization; preserveAspect; endGroup]
*)
networkconnectivity = { { 0, 0.25, 0, .75},
					    { 0, 0, 1, 0},
					    { 0.15, 0.45, 0, 0.40},
			            { 0, 0, 0, 0} };
			         
lwlambda = {20,30,0,0};		
mu = {40,100,150,55};   
numberofserver = {1,1,1,1};
(*
:[font = subsection; inactive; preserveAspect; startGroup]
Example 2 -
:[font = input; locked; preserveAspect; endGroup]
networkconnectivity = { { 0, 1/4 },
					    { 1/5, 0 } };			      
lwlambda = {90,25};		
mu = {150,100};   
numberofserver = {1,1};
:[font = subsection; inactive; locked; preserveAspect; startGroup]
User Input Cell
:[font = input; preserveAspect; endGroup]

networkconnectivity = { };
lwlambda = { };
mu = { };
numberofserver = { };
:[font = subsection; inactive; locked; preserveAspect; startGroup]
Graphical Representation of Jackson Network
:[font = text; inactive; locked; preserveAspect]
The network which has been input into the system can be viewed here.  This is provided as an aid so that one can see if the system which was input matches what was expected.  The Mathematica code to implement computenetwork is contained in the package digraph.m. The Mathematica notebook interface seems to contain bugs in its graphical display functions.  The computegraph function might require multiple execution to generate the correct graphical output.
;[s]
2:0,0;361,1;457,-1;
2:1,0,0 ,times,0,12,0,0,0;1,0,0 ,times,1,12,0,0,0;
:[font = input; locked; initialization; preserveAspect; startGroup]
*)
computenetwork [ networkconnectivity, lwlambda ]
(*
:[font = postscript; PostScript; formatAsPostScript; output; inactive; preserveAspect; pictureLeft = 31; pictureTop = 27.4375; pictureWidth = 391; pictureHeight = 241.625]
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.397542 0.0861538 0.372339 0.053246 [
[(0)] .30045 .16798 0 0 Msboxa
[(1)] .7282 .31233 0 0 Msboxa
[(0.25)] .5574 .26678 0 0 Msboxa
[(0)] .30045 .16798 0 0 Msboxa
[(3)] .06689 .43234 0 0 Msboxa
[(0.75)] .14059 .27354 0 0 Msboxa
[(1)] .7282 .31233 0 0 Msboxa
[(2)] .49463 .5767 0 0 Msboxa
[(1)] .56834 .41789 0 0 Msboxa
[(2)] .49463 .5767 0 0 Msboxa
[(0)] .30045 .16798 0 0 Msboxa
[(0.15)] .35447 .34572 0 0 Msboxa
[(2)] .49463 .5767 0 0 Msboxa
[(1)] .7282 .31233 0 0 Msboxa
[(0.45)] .65449 .47114 0 0 Msboxa
[(2)] .49463 .5767 0 0 Msboxa
[(3)] .06689 .43234 0 0 Msboxa
[(0.4)] .23768 .4779 0 0 Msboxa
[(20)] .2149 .05195 0 0 Msboxa
[(30)] .91594 .31271 0 0 Msboxa
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
[ ] 0 setdash
0 g
p
P
% Start of user PostScript
/mathtops {
gsave
MBeginOrig
moveto
MEndOrig
currentpoint
grestore
} bind def
/MAtocoords {
mathtops 4 2 roll mathtops
4 copy pop pop
3 -1 roll sub
/arry exch def
exch sub
/arrx exch def
arrx dup mul
arry dup mul
add sqrt
/arrl exch def
translate
} bind def
/MAarrowhead1 {
gsave
MAtocoords
[ arrx arrl div
arry arrl div
-1 arry mul arrl div
arrx arrl div
0 0 ]
concat
-0.05 0.0125 moveto 0 0 lineto -0.05 -0.0125 lineto fill
-0.05 0.0125 moveto 0 0 lineto -0.05 -0.0125 lineto
-0.05 0.0125 lineto stroke
grestore
} def
% End of user PostScript
p
p
.004 w
newpath
matrix currentmatrix
.04308 .02662 scale
6.97473 6.30958 1 0 365.73 arc
setmatrix
s
[(0)] .30045 .16798 0 0 Mshowa
newpath
matrix currentmatrix
.04308 .02662 scale
16.9046 11.7316 1 0 365.73 arc
setmatrix
s
[(1)] .7282 .31233 0 0 Mshowa
p
.33826 .18074 m
.69039 .29958 L
s
% Start of user PostScript
-0.688091 -3.59835 3.39913 -1.36655 MAarrowhead1
% End of user PostScript
P
[(0.25)] .5574 .26678 0 0 Mshowa
P
p
.004 w
newpath
matrix currentmatrix
.04308 .02662 scale
6.97473 6.30958 1 0 365.73 arc
setmatrix
s
[(0)] .30045 .16798 0 0 Mshowa
newpath
matrix currentmatrix
.04308 .02662 scale
1.5528 16.2393 1 0 365.73 arc
setmatrix
s
[(3)] .06689 .43234 0 0 Mshowa
p
.27981 .19135 m
.08753 .40898 L
s
% Start of user PostScript
-1.36655 -3.39913 -3.59835 0.688091 MAarrowhead1
% End of user PostScript
P
[(0.75)] .14059 .27354 0 0 Mshowa
P
p
.004 w
newpath
matrix currentmatrix
.04308 .02662 scale
16.9046 11.7316 1 0 365.73 arc
setmatrix
s
[(1)] .7282 .31233 0 0 Mshowa
newpath
matrix currentmatrix
.04308 .02662 scale
11.4825 21.6617 1 0 365.73 arc
setmatrix
s
[(2)] .49463 .5767 0 0 Mshowa
p
.70755 .3357 m
.51528 .55333 L
s
% Start of user PostScript
3.59835 -0.688091 1.36655 3.39913 MAarrowhead1
% End of user PostScript
P
[(1)] .56834 .41789 0 0 Mshowa
P
p
.004 w
newpath
matrix currentmatrix
.04308 .02662 scale
11.4825 21.6617 1 0 365.73 arc
setmatrix
s
[(2)] .49463 .5767 0 0 Mshowa
newpath
matrix currentmatrix
.04308 .02662 scale
6.97473 6.30958 1 0 365.73 arc
setmatrix
s
[(0)] .30045 .16798 0 0 Mshowa
p
.4825 .55115 m
.31259 .19353 L
s
% Start of user PostScript
0.986064 3.35823 -0.986064 -3.35823 MAarrowhead1
% End of user PostScript
P
[(0.15)] .35447 .34572 0 0 Mshowa
P
p
.004 w
newpath
matrix currentmatrix
.04308 .02662 scale
11.4825 21.6617 1 0 365.73 arc
setmatrix
s
[(2)] .49463 .5767 0 0 Mshowa
newpath
matrix currentmatrix
.04308 .02662 scale
16.9046 11.7316 1 0 365.73 arc
setmatrix
s
[(1)] .7282 .31233 0 0 Mshowa
p
.51528 .55333 m
.70755 .3357 L
s
% Start of user PostScript
1.36655 3.39913 3.59835 -0.688091 MAarrowhead1
% End of user PostScript
P
[(0.45)] .65449 .47114 0 0 Mshowa
P
p
.004 w
newpath
matrix currentmatrix
.04308 .02662 scale
11.4825 21.6617 1 0 365.73 arc
setmatrix
s
[(2)] .49463 .5767 0 0 Mshowa
newpath
matrix currentmatrix
.04308 .02662 scale
1.5528 16.2393 1 0 365.73 arc
setmatrix
s
[(3)] .06689 .43234 0 0 Mshowa
p
.45682 .56394 m
.10469 .4451 L
s
% Start of user PostScript
0.688091 3.59835 -3.39913 1.36655 MAarrowhead1
% End of user PostScript
P
[(0.4)] .23768 .4779 0 0 Mshowa
P
p
p
p
.004 w
.22764 .01472 m
.28832 .14244 L
s
% Start of user PostScript
-1.97213 -6.71645 -1.2678 -4.31772 MAarrowhead1
% End of user PostScript
P
[(20)] .2149 .05195 0 0 Mshowa
P
p
p
.004 w
.97619 .26733 m
.76953 .30483 L
s
% Start of user PostScript
6.71645 -1.97213 4.31772 -1.2678 MAarrowhead1
% End of user PostScript
P
[(30)] .91594 .31271 0 0 Mshowa
P
P
P
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd

:[font = output; output; inactive; preserveAspect; endGroup; endGroup; endGroup]
Graphics["<<>>"]
;[o]
-Graphics-
:[font = section; inactive; locked; preserveAspect; startGroup]
Jackson Network Solution         
:[font = text; inactive; locked; preserveAspect]
The function networksolve solves a Jackson network.  The input parameters are the network connectivity matrix, the external input lambda vector, the server utilization vector, and the number of servers per node vector.  Networksolve outputs the following information: 
	1) Probability of a customer leaving at node x,
	2) The effectival arrival rates at node x,
	3) The probability of one customer in the system,
	4) A list of queue information for each node x.

The following cell contains the Mathematica code to implement networksolve.  Initially this cell is hidden when the notebook is opened.
:[font = input; locked; initialization; closed; preserveAspect]
*)

p[i_, c_, a_, pp0_] := 

	Module[{},
	
		If[Not[NumberQ[i]], 
	    	Return[{pp0 a^i /i!, pp0 a^i/c!/c^(i-c)}], 
			If[i <= c,  
				Return[{pp0 a^i /i! }], 
				Return[{pp0 a^i/c!/c^(i-c)}]
		  ]
	  ]
]
(*
:[font = input; locked; initialization; closed; preserveAspect]
*)
calceffectivearrivalrate[ netcon_, lwlambda_]:=

	Module[ { pmatrix, i, b, uplambda },
	
		pmatrix = netcon;
		
		Do [ pmatrix [[ i, i ]] = pmatrix [[ i, i ]] - 1, 
		     { i, 1, Length[pmatrix] } 
		];
			
		pmatrix = Transpose [ pmatrix ];
		b = lwlambda;
		
		Do [ b[[i]] = - b[[i]], 
		     { i, 1, Length[b] }
		];
		
		uplambda = LinearSolve[pmatrix, b];
		Return[uplambda]
	]
 
(*
:[font = input; locked; initialization; closed; preserveAspect]
*)
   
cca[ c_, aaa_ ] := 

	Module [ {}, 

		Return [ 1/((c!(1-(aaa/c))/aaa^c) 
				 ( Sum [ aaa^i/i! , {i,0,c-1} ] ) + 1) 
		]	
	]
(*
:[font = input; locked; initialization; closed; preserveAspect]
*)
solvemmc[inode_, lambda_, muu_, numofservers_, n_ ]:=

	Module[{mylist, tmpstr},
	
		mylist = Simplify [ Solve [ {
			a == lambda / muu,
			lenq == a cca[numofservers, a] / ( numofservers - a ),
			ws == 1 / muu,
			wq == lenq / lambda,
			w == wq + ws,
			len == lambda w,
			lens == len - lenq },
			{ a, lenq, len, lens, w, ws, wq },
			{ a, lenq, len, lens, w, ws, wq }]
		];
		mylist = Sort[ Flatten[ mylist ]];
		
		p0 = Simplify[1 / (mylist[[1,2]]^numofservers/numofservers!/(1-mylist[[1,2]]/numofservers)+
					  Sum[(mylist[[1,2]]^i)/(i!),{i,0,numofservers-1}])];
		pn = Simplify[p[n, numofservers, mylist[[1,2]], p0]];
		
		tmpstr = StringJoin["Node ", ToString[inode]];
		mylist = Prepend[mylist, tmpstr];
		mylist = Append [ mylist, "p0 - >"];
		mylist = Append [ mylist, p0 ];
		(*tmpstr = StringJoin["p0 -> ", ToString[p0]];
		mylist = Append [ mylist, tmpstr];*)  
		mylist = Append [ mylist, "pn - >"];
		mylist = Append [ mylist, pn ];
		
		Return[mylist];
	]
(*
:[font = input; locked; initialization; closed; preserveAspect]
*)
networksolve [ netcon_, lwlambda_, mulist_, numbservlist_, n_ ] :=

	Module [ {uplambda, size, nodelist={}, invalidflag, poneinsystem,i,j,tmp},

		size = Length [ netcon ];
        invalidflag = 0;
        
		uplambda = calceffectivearrivalrate[netcon, lwlambda];
		
		Do [
			If [ uplambda [[i]] > mulist[[i]] numbservlist[[i]],
			     invalidflag = 1,
			     {}
			],
			{ i, 1, Length[ uplambda ] }
	    ];
	      
	    If [ invalidflag == 0,
	    {
			q = Table [ 0, { i, 1, size } ];
		
			Do[  q[[i]] = 1 - Sum[netcon[[i,k]], {k,1,size}], 
			     {i,1,size}
			];
	
			Do[ nodelist = Append [nodelist, 
								   solvemmc[i, uplambda[[i]], mulist[[i]], numbservlist[[i]], n ] ],
		 	   {i, 1, size}
			];
			
			poneinsystem = 0;
			Do[
			{
				tmp = 1;
				Do[
{	
					If[i==j, tmp = tmp * nodelist[[j,12,1]] /. n -> 1, tmp = tmp * nodelist[[j,10]] ];				
				},
				{j,1,size}];
				poneinsystem = poneinsystem + tmp;
			},
			{i,1,size}
			];	
    
			Print["The probability of a customer leaving the network is: ", q];
			Print["The effective arrival rates are: ", Simplify[uplambda]];
			Print["The probability of one customer in the network is: ", poneinsystem];
			Print[TableForm[nodelist] ];
			Print[""];
			Print["Note: pn is a list. The first element is the probability of n customer in the node"];
			Print["when n is less than or equal to the number of server and second element is the"];
			Print["probability of n customer in the node when n is greater than or equal to the number"];
			Print["of server."];	
		},
		{
			Print["This network does not have a steady state solution because"];
			Print["one or more effective arrival rates are greater than a node's"];
			Print["number of servers times its departure rate."]
		}
		]
		Return;
] 		         
(*
:[font = input; locked; initialization; preserveAspect; endGroup; endGroup]
*)
networksolve [ networkconnectivity, lwlambda, mu, numberofserver, n]
(*
^*)