BeginPackage [ "queue`" ]

cca::usage="cca [ c, a ] returns the value of Erlang's Delay formula, 
given c ( the number of servers ), and a ( the server utilization."

bca::usage="bca [ c, a ] returns the value of Erlang's Loss formula
given c ( the number of servers ), and a ( the server utilization )."

mm1lenplot::usage="mm1lenplot [ lambdamax, mumax ] computes a plot
of Lq, Ls, and L given a lambda max and mu max for the M/M/1 queueing system."

mm1waitplot::usage="mm1waitplot [ lambdamax, mumax ] computes a plot
of Wq, Ws, and W given a lambda max and mu max for the M/M/1 queueing system."

mm1klenplot::usage="mm1klenplot [ lambdamax, mumax ] computes a plot 
of Lq, and L given a maxmimum lambda, a maximum mu, and k ( the maximum number
of customers in the system ) for the M/M/1/K queueing system."

mm1kwaitplot::usage="mm1kwaitplot [ lamdbamax, mumax ] computes a plot of
Wq and W given a maximum lambda, a maximum mu and k ( the maximum number
of customers allowed in the system ) for the M/M/1/K queueing system."

mmclenplot::usage="mmclenplot [ servermax ] computes a plot of Lq and L 
given the maximum number of servers in the system for the M/M/c queuing
system."

mmcwaitplot::usage="mmcwaitplot [ servermax ] computes a plot of Wq and W
given the maximum number of servers in the system for the M/M/c queuing
system."

mmcclenplot::usage="mmcclenplot [ servermax ] computes a plot of L 
given the maximum number of servers for a M/M/c/c queuing system."

mm1kklenplot::usage="mm1kklenplot [ lambdamax, kmax, mttf ] computes a plot
of L given a maximum lambda, maximum number of customers k, and a mean
time to failure, mttf for the M/M/1/K/K queueing system."

mm1kkwaitplot::usage="mm1kkwaitplot [ lambdamax, mu, kmax, mttf ] computes
a plot of W and Wq given a maximum lambda, mu, maximum number of
customers k, and a mean time to failure for the M/M/1/K/K queueing system." 

ccaplot::usage="cca [ servermax ] computes a plot of Erlang's Delay
formula given a maximum number of servers."

bcaplot::usage="bca [ servermax ] computes a plot of Erlang's Loss
formula given a maximum number of servers."

Begin["`Private`"]

cca [c_,a_] := Module [ {},
	Return[((c!(1-(a/c))/a^c)(Sum[a^i/i!,{i,0,c-1}])+1)^-1]
]

bca [c_,a_] := Module[ {},
	Return[Simplify[((a^c)/c!)/Sum[ a^i / i!, {i,0,c}]]]
]

mm1lenplot [ lambdamax_, mumax_ ] := Module [ {f,g,h},

  f [ lam_, mu_ ] := If [ lam >= mu,
	        	  Return[0],
			  Return[lam/(mu-lam)] 
		     ];

  g [ lam_, mu_ ] := If [ lam >= mu,
	     		  Return[0],
	     		  Return[lam/mu]
	     	     ];

  h [ lam_, mu_ ] := If [ lam >= mu,
	      	 	  Return[0],
	   	 	  Return[(lam^2)/(mu(mu-lam))]   	 
	   	     ];

  Show [ Plot3D[ f [ lam,mu ] ,{lam,0,lambdamax},{mu,0,mumax},
	         ViewPoint -> {-1,2,1}, Axes -> True,
	         AxesLabel -> { "Lambda", "Mu", "Total Number in System" } 
         ],
         Plot3D[ g [ lam,mu ] ,{lam,0,lambdamax},{mu,0,mumax},
                 ViewPoint -> {-1,2,1}, Axes -> True,
                 AxesLabel -> { "Lambda", "Mu", "Total Number in Server" }
	 ],
	 Plot3D[ h [ lam,mu ] ,{lam,0,lambdamax},{mu,0,mumax},
	         ViewPoint -> {-1,2,1}, Axes -> True,
	         AxesLabel -> { "Lambda", "Mu", "Total Number in Queue" }
         ]
  ]
]

mm1waitplot [ lambdamax_, mumax_ ] := Module [ {f,g,h},

  f [ lam_, mu_ ] := If [ lam >= mu,
			  Return [0],
			  Return[ (1/mu)/(1-lam/mu) ]
		     ];

  g [ lam_, mu_ ] := If [ lam >= mu,
			  Return [0],
			  Return[ 1/mu ]
			];

  h [ lam_, mu_ ] := If [ lam >= mu,
			  Return [0],
			  Return[ (lam/mu)(1/mu)/(1-lam/mu) ]
			];

  Show [ Plot3D[ h [ lam,mu ] ,{lam,0,lambdamax},{mu,0,mumax},
		 ViewPoint -> {-1,2,1}, Axes -> True,
	         AxesLabel -> { "Lambda", "Mu", "Wait Time in Queue" }
               ],
	 Plot3D[ g [ lam,mu ] ,{lam,0,lambdamax},{mu,0,mumax},
	         ViewPoint -> {-1,2,1}, Axes -> True,
		 AxesLabel -> { "Lambda", "Mu", "Wait Time in Server" }
               ],
	 Plot3D[ f [ lam,mu ] ,{lam,0,lambdamax},{mu,0,mumax},
		 ViewPoint -> {-1,2,1}, Axes -> True,
		 AxesLabel -> { "Lambda", "Mu", "Total Wait Time in System" }
	       ]
  ]
]

mm1klenq [ lam_, mu_, k_ ] := Module [ {l,p0},
	
    l = mm1klength [ lam, mu, k ];
    p0 = mm1kpn [ lam, mu, k, 0 ];
    Return[ l - ( 1 - p0 ) ]

  ];

mm1klength [ lam_, mu_, k_ ] := Module [ {a},

    a = lam/mu;

    If [ lam == mu,
         Return [ k/2 ],
         Return [ a ( 1 - ( k + 1 ) a^k + k a^(k+1) ) /
                  ( ( 1 - a ) ( 1 - a^(k+1) ) ) ]
       ]
  ];

mm1kpn [ lam_, mu_, k_, n_ ] := Module [ {a},

    a = lam / mu;

    If [ lam == mu,
         Return [ 1 / ( k + 1 ) ],
         Return [ ( 1 - a ) a^n / ( 1 - a^(k+1) ) ]
       ]
  ];

mm1klenplot [ lambdamax_, mumax_, kk_ ] := Module [ {},

  Show [ Plot3D[ mm1klength [ lam, mu, kk ], {lam,0.01,lambdamax}, {mu,0.01,mumax},
	         ViewPoint -> {-1,2,1}, Axes -> True,
                 AxesLabel -> {"Lambda", "Mu", "Total Number in System" }
               ],
         Plot3D[ mm1klenq [ lam, mu, kk ], {lam, 0.01, lambdamax}, {mu,0.01,mumax},
                 ViewPoint -> {-1,2,1}, Axes -> True,
                 AxesLabel -> {"Lambda", "Mu", "Total Number in Queue" }
               ]
  ]
] 

mm1kwaitplot [ lambdamax_, mumax_, kk_ ] := Module [ {lambdaa},

  lambdaa [ lam_, mu_, k_ ] := ( 1 - mm1kpn [ lam, mu, k, k ] ) lam;

  Show [ Plot3D[ mm1klength [ lam, mu, kk ] / lambdaa [ lam, mu, kk ],
                 {lam, 0.01, lambdamax}, {mu, 0.01, mumax},
                 ViewPoint -> { -1, 2, 1}, Axes -> True,
                 AxesLabel -> {"Lambda","Mu", "Total Wait in System" }
               ],
         Plot3D[ mm1klenq [ lam, mu, kk ] / lambdaa [ lam, mu, kk ],
                 {lam, 0.01, lambdamax}, {mu, 0.01, mumax},
                 ViewPoint -> { -1, 2, 1}, Axes -> True,
                 AxesLabel -> {"Lambda","Mu", "Total Wait in Queue" }
               ]
  ]
]


mmclenq [ rho_, serv_ ] := Module [ {},

  Return [ rho cca [ serv, rho ] / ( 1 - rho ) ]
]


mmclength [ rho_, serv_ ] := Module [ {lambda},
 
  lambda = 10;
  Return [ mmcw [ rho, serv ] lambda ]
]


mmcwq [ rho_, serv_ ] := Module [ {lambda},
  
  lambda = 10;
  Return [ mmclenq [ rho, serv ] / lambda ]
]


mmcw [ rho_, serv_ ] := Module [ {lambda},

  lambda = 10;
  mu = lambda / rho;
  Return [ 1/mu + mmcwq [ rho, serv ] ]
]


mmclenplot [ servermax_ ] := Module [ {lambda, rho},

  Show [ Plot3D [ mmclenq [ rho, serv ], {rho,0.01,.99}, 
                  {serv, 1, servermax},
                  ViewPoint -> {-1,2,1}, Axes -> True,
                  AxesLabel -> {"Lambda/mu", "Servers", "Total Number in Queue"}
                ],
         Plot3D [ mmclength [ rho, serv ], {rho, 0.01, .99},
                  {serv, 1, servermax},
                  ViewPoint -> {-1,2,1}, Axes -> True,
                  AxesLabel -> {"Lambda/mu", "Servers", "Total Number in System"}
                ]
       ]
]


mmcwaitplot [ servermax_ ] := Module [ { lambda, rho },

  Show [ Plot3D [ mmcwq [ rho, serv ], {rho, 0.01, .99},
                  {serv, 1, servermax},
                  ViewPoint -> {-1,2,1}, Axes -> True,
                  AxesLabel -> {"Lambda/mu", "Servers", "Total Wait in Queue"}
                ],
         Plot3D [ mmcw [ rho, serv ], {rho, 0.01, .99},
                  {serv, 1, servermax},
                  ViewPoint -> {-1,2,1}, Axes -> True,
                  AxesLabel -> {"Lambda/mu", "Servers", "Total Wait in System"}
                ]
  ]
]


mmcclenplot [ servermax_ ] := Module [ {mu, lambda},

  lambda = 20;

  lambdaa [ rho_, serv_ ] := lambda ( 1 - bca [ serv, rho ] );
  mu [ rho_ ] := lambda / rho;

  Plot3D [ lambdaa [ rho, serv ] 1/mu[rho], {rho,0.01,0.99},  
           {serv, 1, servermax}, 
           ViewPoint -> {-4,-1,1}, Axes -> True,
           AxesLabel -> {"Lambda", "Mu", "Total Number in System"}
         ]
]


mm1kkw [ k_, lam_, mttf_ ] := Module [ {},

  Return [ k/lam - mttf ]
]


mm1kkwq [ k_, lam_, mu_, mttf_ ] := Module [ {},
 
  Return [ mm1kkw [ k, lam, mttf ] - 1/mu ]
]


mm1kklenplot [ lambdamax_, kk_, mttf_ ] := Module [ {},

  Plot3D[ lam mm1kkw [ k, lam, mttf ], {k,1,kk}, {lam,0.01,lambdamax},
           ViewPoint -> {1,-2,1}, Axes -> True,
           AxesLabel -> {"K", "Lambda", "Total Number in System"}
         ]
]


mm1kkwaitplot [ lambdamax_, mu_, kk_, mttf_ ] := Module [ {},

  Plot3D[ mm1kkw[ k, lam, mttf ] - 1/mu, {k,1,kk},{lam,0.01,lambdamax},
           ViewPoint -> {-4,2,1}, Axes -> True,
           AxesLabel -> {"K", "Lambda", "Total Wait in Queue"}
         ];

  Plot3D[ mm1kkw [ k, lam, mttf ], {k,1,kk}, {lam,0.01,lambdamax},
           ViewPoint -> {-4,2,1}, Axes -> True,
           AxesLabel -> {"K", "Lambda", "Total Wait in System"}
         ]
]


ccaplot [ servermax_ ] := Module [ {},

  Plot3D [ cca [ c, a ], { c, 1, servermax }, { a, 0.01, 0.99},
           ViewPoint -> {1,-2,1}, Axes -> True,
           AxesLabel -> {"Servers", "Lambda/Mu", "C[c,a]"}
         ]
]


bcaplot [ servermax_ ] := Module [ {},
  Plot3D [ bca [ c, a ], { c, 1, servermax }, { a, 0.01, 0.99},
           ViewPoint -> {1,-2,1}, Axes -> True,
           AxesLabel -> {"Servers", "Lambda/Mu", "B[c,a]"}
         ]
]



